@@ -17,17 +17,22 @@ import re
 import collections
 import os
 import yaml
+import shutil
 from jinja2 import Template
 from copy import deepcopy
 import requests
 from cvapianalyser import CommunityEdition
 from openapispecdiff import OpenApiSpecDiff
 
+import urllib3
+urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
+
 root = os.path.dirname(os.path.abspath(__file__))
 templates_dir = os.path.join(root, 'templates')
-fuzz_words_dir = os.path.join(root, 'wfuzz/wordlist')
+fuzz_words_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'wfuzz/wordlist')
 fuzz_types = ["general", "injections", "vulns", "webservices", "stress", "others"]
 ANOMOLY_THRESHOLD = 1.07142
+RESERVED_PARAMETERS = ["$RESPONSE_ERROR"]
 
 
 def keypaths(nested):
@@ -62,16 +67,21 @@ class CloudvectorDAST(object):
         self.cv_config = None
         self.cvdast_to_auth_info = {}
         self.params_from_spec = {}
+        self.respcodes_from_spec = {}
         self.params_details_from_spec = {}
         if config_file:
             if os.path.exists(config_file):
                 self.cv_config = self._parse_cv_config(config_file)
                 ce_host = self.cv_config["ce_setup"]["ce_host"]
                 ce_username = self.cv_config["ce_setup"]["ce_username"]
-                custom_validations = self.cv_config.get("custom_validations")
+                custom_validations = self.cv_config.get("custom_validations",{})
                 self.static_headers_from_cfg = self.cv_config.get("static_headers")
                 if self.cv_config.get("authentication"):
                   self.cvdast_to_auth_info = self.cv_config.get("authentication")
+        if str(self.cv_config.get("generate_tests_for_response_codes","Y")).lower() == "y":
+            self.generate_tests_for_respcodes = True
+        else:
+            self.generate_tests_for_respcodes = False
         if APISpecOne:
             self.apispec_one_path = APISpecOne
         else:
@@ -85,7 +95,17 @@ class CloudvectorDAST(object):
         self.input_json = {}
         if not os.path.exists("tests"):
             os.mkdir("tests")
-        self.prepped_spec = self._prepare_spec_for_test_generation(self.apispec_two_path)
+        if do_fuzz:
+            if not os.path.exists((os.path.join(os.getcwd(),"wfuzz"))):
+                shutil.copytree(os.path.join(os.path.dirname(os.path.abspath(__file__)),"wfuzz"), os.path.join(os.getcwd(),"wfuzz"))
+        self.params_captured_in_traffic = {}
+        changed_apis = self._get_changed_apis()
+        apis_to_check = changed_apis["changed"]
+        apis_to_check.update(changed_apis["new"])
+        if self.apispec_one_path:
+            self.prepped_spec = self._prepare_spec_for_test_generation(self.apispec_two_path, changed_apis)
+        else:
+            self.prepped_spec = self._prepare_spec_for_test_generation(self.apispec_two_path)
         self.input_json = {}
         self.assertions_map = {}
         for file in input_params_file.split(";"):
@@ -99,10 +119,7 @@ class CloudvectorDAST(object):
                     input_json = self._load_input_from_files(file)
             self.input_json.update(input_json)
 
-        self.params_captured_in_traffic = {}
-        changed_apis = self._get_changed_apis()
-        apis_to_check = changed_apis["changed"]
-        apis_to_check.update(changed_apis["new"])
+
 
         cv_events = self._process_event_data(apis_to_check)
         # if not cv_events:
@@ -172,7 +189,7 @@ class CloudvectorDAST(object):
     #     with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), "tests/authcreds.json"),"w+") as fobj:
     #         json.dumps(auth_info)
 
-    def _prepare_spec_for_test_generation(self, input_spec):
+    def _prepare_spec_for_test_generation(self, input_spec, specific_apis=None):
         parsed_new_spec = self._get_spec_parsed(input_spec)
         apis_from_spec = []
         if "servers" in parsed_new_spec:
@@ -188,6 +205,11 @@ class CloudvectorDAST(object):
         if type(baseurl) is list:
             baseurl = baseurl[0]
         for api, info in parsed_new_spec["paths"].items():
+            if specific_apis:
+                if api not in specific_apis:
+                    continue
+            if api not in self.respcodes_from_spec:
+                self.respcodes_from_spec[api] = {}
             for method, minfo in info.items():
                 x = {}
                 x["method"] = method.lower()
@@ -208,19 +230,23 @@ class CloudvectorDAST(object):
                         _ = str(_).replace("{", "").replace("}", "")
                         x["params"].append(_)
                 #x["params"] += url_params
+                if x["http-req-url"] not in self.respcodes_from_spec:
+                    self.respcodes_from_spec[x["http-req-url"]] = {}
+                self.respcodes_from_spec[x["http-req-url"]][method] = list(minfo.get("responses").keys())
                 x["url"] = x["url"].replace("{", "{{").replace("}", "}}")
                 for _ in minfo.get("parameters", []):
-                    if _["in"] == "header":
-                        x["header"][_["name"]] = ""
-                    elif _["in"] == "query":
+                    if _.get("in") == "header":
+                        x["header"][_.get("name")] = ""
+                    elif _.get("in") == "query":
                         if "?" not in x["url"]:
-                            x["url"] += "?" + _["name"] + "={{" + _["name"] + "}}"
+                            x["url"] += "?" + _.get("name") + "={{" + _.get("name") + "}}"
                         else:
-                            x["url"] += "&" + _["name"] + "={{" + _["name"] + "}}"
-                        x["params"].append(_["name"])
+                            x["url"] += "&" + _.get("name") + "={{" + _.get("name") + "}}"
+                        x["params"].append(_.get("name"))
                     else:
-                        x["params"].append(_["name"])
-                        x["body"][_["name"]] = ""
+                        if _.get("name"):
+                            x["params"].append(_.get("name"))
+                            x["body"][_.get("name")] = ""
                     if _.get("value"):
                         if type(_.get("value")) is list:
                             for e in _.get("value"):
@@ -253,6 +279,8 @@ class CloudvectorDAST(object):
             host = input_spec.get("host")
         if type(baseurl) is list:
             baseurl = baseurl[0]
+        if baseurl == "/":
+            baseurl = ""
         params_info = {}
         for api, info in input_spec["paths"].items():
             api = baseurl+api
@@ -269,7 +297,7 @@ class CloudvectorDAST(object):
         return params_info
 
     def _get_spec_parsed(self, input_path):
-        return self.openapispec_obj.scan_input_spec(input_path)
+        return self.openapispec_obj.scan_input_spec(self.openapispec_obj.parse_spec(input_path))
 
     def _get_changed_apis(self):
         return self.openapispec_obj.diff
@@ -313,7 +341,7 @@ class CloudvectorDAST(object):
         if os.path.isdir(os.path.join(fuzz_words_dir, fuzz_type)):
             for (root, dirs, files) in os.walk(os.path.join(fuzz_words_dir, fuzz_type), topdown=True):
                 for _file in files:
-                    fuzz_values.append({"file": os.path.join(os.path.join("../wfuzz/wordlist/", fuzz_type), _file),
+                    fuzz_values.append({"file": os.path.join(os.path.join("wfuzz/wordlist/", fuzz_type), _file),
                                         "fuzz_type": fuzz_type})
                     # values = []
                     # with open(file) as _file:
@@ -342,9 +370,9 @@ class CloudvectorDAST(object):
         auth_api_response_key = None
         auth_inputs = ["host"]
         if self.cvdast_to_auth_info:
-            auth_api = self.cvdast_to_auth_info["api_url"]
-            auth_api_headers = self.cvdast_to_auth_info["headers"]
-            auth_api_payload = self.cvdast_to_auth_info["payload"]
+            auth_api = self.cvdast_to_auth_info.get("api_url")
+            auth_api_headers = self.cvdast_to_auth_info.get("headers")
+            auth_api_payload = self.cvdast_to_auth_info.get("payload")
             for k, v in auth_api_headers.items():
                 template_found = re.search(r"\[\'([A-Za-z0-9_]+)\'\]", str(v))
                 if template_found:
@@ -355,13 +383,22 @@ class CloudvectorDAST(object):
                 if template_found:
                     auth_inputs.append(template_found.group(1))
                     auth_api_payload[k] = auth_inputs[-1]
-            auth_api_response_key = self.cvdast_to_auth_info["response_key"]
+            auth_api_response_key = self.cvdast_to_auth_info.get("response_key")
+            auth_type = self.cvdast_to_auth_info.get("type", "basic").lower()
+            if auth_type == "basic":
+                auth_prefix = ""
+            elif auth_type == "bearer":
+                auth_prefix = "Bearer"
         for api,params_info in self.params_from_spec.items():
-            if api not in self.params_captured_in_traffic:
-                self.params_captured_in_traffic[api] = {}
-            for k, v in params_info.items():
-                if k not in self.params_captured_in_traffic[api]:
-                    self.params_captured_in_traffic[api][k]=v
+            for statuscode in self.params_captured_in_traffic:
+                if int(statuscode) not in range(200,299):
+                    if not self.params_captured_in_traffic.get(statuscode):
+                        continue
+                    if api not in self.params_captured_in_traffic[statuscode]:
+                        self.params_captured_in_traffic[statuscode][api] = {}
+                    for k, v in params_info.items():
+                        if k not in self.params_captured_in_traffic[statuscode].get(api,{}):
+                            self.params_captured_in_traffic[statuscode][api][k]=v
         nested_params={}
         more_params = []
         for param, info in self.params_details_from_spec.items():
@@ -373,11 +410,26 @@ class CloudvectorDAST(object):
                     tmp.append(k)
             nested_params[param] = tmp
             more_params+=tmp
+
+        params_for_fixtures = []
+        for k, v in self.params_captured_in_traffic.items():
+            for api, plist in v.items():
+                #print(api, plist)
+                for each in plist:
+                    if type(each) is str:
+                        if each not in params_for_fixtures and each not in RESERVED_PARAMETERS:
+                            params_for_fixtures.append(each)
+                    elif type(each) is dict:
+                        for _ in each.keys():
+                            if _ not in params_for_fixtures and _ not in RESERVED_PARAMETERS:
+                                params_for_fixtures.append(_)
+
+
         with open("tests/conftest.py", 'w+') as fh:
-            fh.write(template.render(api_info=self.params_captured_in_traffic, api_detailed_info=self.params_details_from_spec,
+            fh.write(template.render(api_info=params_for_fixtures, api_detailed_info=self.params_details_from_spec,
                                      nested_params=nested_params, AUTH_API=auth_api, more_params=more_params,
                                      AUTH_API_HEADERS=auth_api_headers,AUTH_API_PAYLOAD=json.dumps(auth_api_payload),
-                                     AUTH_RESP_KEY=auth_api_response_key, AUTH_INPUTS=auth_inputs))
+                                     AUTH_RESP_KEY=auth_api_response_key, AUTH_INPUTS=auth_inputs, TOKEN_PREFIX=auth_prefix))
         print("\n\t\t......done creating pytest fixtures (conftest.py)")
 
     def create_fuzzfixtures(self):
@@ -413,6 +465,8 @@ class CloudvectorDAST(object):
         if url not in self.assertions_map:
             url = url.replace("/", "_").replace("-", "_").replace("{", "").replace("}", "").split("?")[0]
             self.assertions_map[url] = {}
+        if None in params:
+            params.remove(None)
 
         for param in params:
             paths_in_req = key_lookup(param, req_payload)
@@ -446,13 +500,122 @@ class CloudvectorDAST(object):
 
     def create_pytest_methods(self, cv_events, custom_validations):
         # print("\n\n\n"+str(custom_validations)+"\n\n\n")
-        params_for_custom_validations = custom_validations["request_params"]
+        params_for_custom_validations = custom_validations.get("request_params")
         apis_to_be_tested = {}
         files_created = set()
         print("\n\ncreating Pytest test methods....\n")
         new_spec_info = self._scan_input_spec(self.apispec_two_path)
         hosts = new_spec_info.get("servers", {}).get("url", [])
         #print(cv_events)
+        for _ in cv_events:
+            #print(_)
+            event = _[0]
+            if _[1]:
+                if int(_[1]["attributes"]["http_rsp_status_code"]) not in range(200, 299):
+                    continue
+            if event["url"]:
+                api = str(event["http-req-url"]).lstrip("/").rstrip("/").replace("/", "_").replace("-", "_").split("?")[
+                    0]
+
+                if api not in apis_to_be_tested:
+                    apis_to_be_tested[api] = {}
+                #apis_to_be_tested[api]["method"] = event["method"]
+                method = event["method"].lower()
+                if method not in apis_to_be_tested[api]:
+                    apis_to_be_tested[api][method] = {}
+                if self.static_headers_from_cfg:
+                    for k, v in event["header"].items():
+                        if k in self.static_headers_from_cfg:
+                            event["header"][k] = str(self.static_headers_from_cfg[k])
+                if "header" not in apis_to_be_tested[api][method]:
+                    apis_to_be_tested[api][method]["header"] = {}
+                apis_to_be_tested[api][method]["header"].update(event["header"])
+                apis_to_be_tested[api][method]["url"] = event["url"]  # str(event["host"]).lower() + event["url"]
+                if apis_to_be_tested[api][method].get("params"):
+                    known_params = apis_to_be_tested[api][method].get("params")
+                else:
+                    known_params = []
+
+                apis_to_be_tested[api][method]["params"] = new_spec_info.get(str(event["http-req-url"]).replace("//", "/"),{}).get(method,[])
+
+                params_in_traffic = event.get("body",{}).keys()
+
+                for _ in params_in_traffic:
+                    if _ not in apis_to_be_tested[api][method]["params"]:
+                        apis_to_be_tested[api][method]["params"].append(_)
+
+                if apis_to_be_tested[api][method]["params"] is None:
+                    apis_to_be_tested[api][method]["params"] = []
+
+                apis_to_be_tested[api][method]["params"] = list(set(apis_to_be_tested[api][method]["params"]))
+                apis_to_be_tested[api][method]["params"].append("host")
+                apis_to_be_tested[api][method]["params"].append("url_prefix")
+                apis_to_be_tested[api][method]["params"].append("access_token")
+
+                host_url = str(event["host"]).lower()
+                hosts.append(host_url)
+                self._create_assertions_map(event["http-req-url"], apis_to_be_tested[api][method]["params"], event["body"],
+                                            event["rsp_body"])
+        with open(os.path.join(templates_dir, 'test_api.j2')) as file_:
+            template = Template(file_.read())
+        #host_url = [list(set([_ for _ in hosts if _ != '']))[-1]]
+
+        for k, v in apis_to_be_tested.items():
+            # code = template.render(api_info=apis_to_be_tested[k], api_name=k, host_url=host_url)
+            filename = str(k).replace("/", "_").replace("{", "").replace("}", "")
+            if not os.path.exists("tests"):
+                os.mkdir("tests")
+            if params_for_custom_validations:
+                extra_validations = self._create_custom_validations(params_for_custom_validations,
+                                                                    apis_to_be_tested[k])
+            else:
+                extra_validations = []
+
+            api_name = "/"+str(k.replace("_","/"))
+            if self.generate_tests_for_respcodes:
+                status_codes_for_api = self.respcodes_from_spec.get(api_name,{})
+            else:
+                status_codes_for_api = {}
+            with open("tests/test_" + str(filename) + ".py", 'w+') as fh:
+                fh.write(template.render(apis_metadata=apis_to_be_tested[k], api_name=k, host_url=host_url,
+                                         custom_validations=extra_validations, STATUS_CODES=status_codes_for_api))
+            files_created.add("test_" + str(filename))
+        print("\n\t\t......done creating pytest methods: " + str(files_created))
+
+    def create_fuzz_test_methods(self, cv_events):
+        apis_to_be_tested = {}
+        files_created = set()
+        print("\n\ncreating Pytest test methods for fuzzing....\n")
+        new_spec_info = self._scan_input_spec(self.apispec_two_path)
+        hosts = new_spec_info.get("servers", {}).get("url", [])
+        # for _ in cv_events:
+        #     event = _[0]
+        #     if _[1]:
+        #         if _[1]["attributes"]["http_rsp_status_code"] not in ["200", "201"]:
+        #             continue
+        #     if event["url"] not in apis_to_be_tested:
+        #         api = str(event["http-req-url"]).lstrip("/").rstrip("/").replace("/", "_").replace("-", "_").split("?")[
+        #             0]
+        #         apis_to_be_tested[api] = {}
+        #         apis_to_be_tested[api]["method"] = event["method"]
+        #         if self.static_headers_from_cfg:
+        #             for k, v in event["header"].items():
+        #                 if k in self.static_headers_from_cfg:
+        #                     event["header"][k] = str(self.static_headers_from_cfg[k])
+        #
+        #         # for k, v in event["header"].items():
+        #         #     if k in self.static_headers_from_cfg:
+        #         #         event["header"][k] = v
+        #         apis_to_be_tested[api]["header"] = event["header"]
+        #         apis_to_be_tested[api]["url"] = event["url"]
+        #         apis_to_be_tested[api]["params"] = new_spec_info.get(str(event["http-req-url"]).replace("//", "/"))
+        #         if apis_to_be_tested[api]["params"] is None:
+        #             apis_to_be_tested[api]["params"] = []
+        #         host_url = str(event["host"]).lower()
+        #         hosts.append(host_url)
+        #         self._create_assertions_map(event["http-req-url"], apis_to_be_tested[api]["params"], event["body"],
+        #                                     event["rsp_body"])
+
         for _ in cv_events:
             event = _[0]
             if _[1]:
@@ -490,70 +653,24 @@ class CloudvectorDAST(object):
                 hosts.append(host_url)
                 self._create_assertions_map(event["http-req-url"], apis_to_be_tested[api][method]["params"], event["body"],
                                             event["rsp_body"])
-        with open(os.path.join(templates_dir, 'test_api.j2')) as file_:
-            template = Template(file_.read())
-        #host_url = [list(set([_ for _ in hosts if _ != '']))[-1]]
-        for k, v in apis_to_be_tested.items():
-            # code = template.render(api_info=apis_to_be_tested[k], api_name=k, host_url=host_url)
-            filename = str(k).replace("/", "_").replace("{", "").replace("}", "")
-            if not os.path.exists("tests"):
-                os.mkdir("tests")
-            extra_validations = self._create_custom_validations(params_for_custom_validations,
-                                                                apis_to_be_tested[k])
-            with open("tests/test_" + str(filename) + ".py", 'w+') as fh:
-                fh.write(template.render(apis_metadata=apis_to_be_tested[k], api_name=k, host_url=host_url,
-                                         custom_validations=extra_validations))
-            files_created.add("test_" + str(filename))
-        print("\n\t\t......done creating pytest methods: " + str(files_created))
 
-    def create_fuzz_test_methods(self, cv_events):
-        apis_to_be_tested = {}
-        files_created = set()
-        print("\n\ncreating Pytest test methods for fuzzing....\n")
-        new_spec_info = self._scan_input_spec(self.apispec_two_path)
-        hosts = new_spec_info.get("servers", {}).get("url", [])
-        for _ in cv_events:
-            event = _[0]
-            if _[1]:
-                if _[1]["attributes"]["http_rsp_status_code"] not in ["200", "201"]:
-                    continue
-            if event["url"] not in apis_to_be_tested:
-                api = str(event["http-req-url"]).lstrip("/").rstrip("/").replace("/", "_").replace("-", "_").split("?")[
-                    0]
-                apis_to_be_tested[api] = {}
-                apis_to_be_tested[api]["method"] = event["method"]
-                for k, v in event["header"].items():
-                    if k in self.static_headers_from_cfg:
-                        event["header"][k] = v
-                apis_to_be_tested[api]["header"] = event["header"]
-                apis_to_be_tested[api]["url"] = event["url"]
-                apis_to_be_tested[api]["params"] = new_spec_info.get(str(event["http-req-url"]).replace("//", "/"))
-                if apis_to_be_tested[api]["params"] is None:
-                    apis_to_be_tested[api]["params"] = []
-                host_url = str(event["host"]).lower()
-                hosts.append(host_url)
-                self._create_assertions_map(event["http-req-url"], apis_to_be_tested[api]["params"], event["body"],
-                                            event["rsp_body"])
-
-        # if len(hosts) > 1:
-        #     host_url = self._get_hosts_input(hosts)
-        # else:
-        #     host_url = hosts[-1]
 
         for type in fuzz_types:
             fuzzing_details = self._get_fuzz_details(type)
             with open(os.path.join(templates_dir, 'fuzz_test.j2')) as file_:
                 template = Template(file_.read())
             for k, v in apis_to_be_tested.items():
-                if not apis_to_be_tested[k]["params"]:
-                    continue
+                #print(k, v)
+                for method, info in v.items():
+                    if not info["params"]:
+                        continue
                 # code = template.render(api_info=apis_to_be_tested[k], api_name=k, host_url=host_url)
                 filename = str(k).replace("/", "_").replace("{", "").replace("}", "") + "_for_" + str(type) + "_fuzzing"
                 if not os.path.exists("tests"):
                     os.mkdir("tests")
 
                 with open("tests/test_" + str(filename) + ".py", 'w+') as fh:
-                    fh.write(template.render(api_info=apis_to_be_tested[k], api_name=k, host_url=host_url,
+                    fh.write(template.render(apis_metadata=apis_to_be_tested[k], api_name=k, host_url=host_url,
                                              fuzzing_details=fuzzing_details, fuzz_type=str(type)))
                 files_created.add("test_" + str(filename))
         print("\n\t\t......done creating pytest methods for fuzzing: " + str(files_created))
@@ -601,6 +718,8 @@ class CloudvectorDAST(object):
         for event in events:
             if "http-req-header-Cv-Fuzzed-Event" in event["attributes"]["event_json"]:
                 continue
+            if int(str(event["attributes"]["http_rsp_status_code"]).split(" ")[0]) == 0:
+                continue
             if apis_to_check:
                 iflag = False
                 # params_to_add = []
@@ -613,7 +732,6 @@ class CloudvectorDAST(object):
 
                 if not iflag:
                     continue
-            # print(str(event["attributes"]["http_path"]))
             request = {"url": str(event["attributes"]["http_path"]), "method": str(event["attributes"]["http_method"]).lower()}
             header = {}
             body = {}
@@ -656,31 +774,56 @@ class CloudvectorDAST(object):
             request["header"] = header
             request["body"] = req_params_found
             request["rsp_body"] = rsp_params_found
-            # if "transfers" in request["http-req-url"]:
-            #     print(request)
-            #     print("\n\n\n")
-            # if not to_skip:
-            # print(event)
-            if str(event["attributes"]["http_path"]).lower() not in self.params_captured_in_traffic:
-                self.params_captured_in_traffic[str(event["attributes"]["http_path"])] = {}
+            #print(event["attributes"]["event_json"])
+            request["rsp_code"] = str(event["attributes"]["http_rsp_status_code"]).split(" ")[0]
+            if request["rsp_code"] not in [200, 201]:
+                request["rsp_error"] = event["attributes"]["event_json"].get("http-rsp-body-error")
+            # if str(request["rsp_code"]) not in self.params_captured_in_traffic:
+            #     self.params_captured_in_traffic[request["rsp_code"]] = {}
+            # if str(event["attributes"]["http_path"]) not in self.params_captured_in_traffic[request["rsp_code"]]:
+            #     self.params_captured_in_traffic[request["rsp_code"]].update( { str(event["attributes"]["http_path"]) : {} } )
+            #
+            # for param, value in req_params_found.items():
+            #     if param in self.params_captured_in_traffic[request["rsp_code"]][str(event["attributes"]["http_path"])]:
+            #         # self.params_captured_in_traffic[str(event["attributes"]["http_path"]).lower()] = {}
+            #         #if value not in self.params_captured_in_traffic[request["rsp_code"]][str(event["attributes"]["http_path"])][
+            #         #    param]:
+            #         self.params_captured_in_traffic[request["rsp_code"]][str(event["attributes"]["http_path"])][param].append(
+            #                 value)
+            #     else:
+            #         self.params_captured_in_traffic[request["rsp_code"]][str(event["attributes"]["http_path"])][param] = [value]
+            #     try:
+            #         self.params_captured_in_traffic[request["rsp_code"]][str(event["attributes"]["http_path"])][param] = \
+            #             list(set(self.params_captured_in_traffic[request["rsp_code"]][str(event["attributes"]["http_path"])][param]))
+            #     except TypeError:
+            #         pass
 
-            # print("\n\n\n\n\n\n\n")
+            if str(request["rsp_code"]) not in self.params_captured_in_traffic:
+                self.params_captured_in_traffic[request["rsp_code"]] = {}
+            if str(event["attributes"]["http_path"]) not in self.params_captured_in_traffic[request["rsp_code"]]:
+                self.params_captured_in_traffic[request["rsp_code"]].update( { str(event["attributes"]["http_path"]) : [] } )
+
+            params_found = {}
             for param, value in req_params_found.items():
-                # if "severity" in param:
-                #     print(str(event) + "\n\n")
-                if param in self.params_captured_in_traffic[str(event["attributes"]["http_path"])]:
-                    # self.params_captured_in_traffic[str(event["attributes"]["http_path"]).lower()] = {}
-                    if value not in self.params_captured_in_traffic[str(event["attributes"]["http_path"])][
-                        param]:
-                        self.params_captured_in_traffic[str(event["attributes"]["http_path"])][param].append(
-                            value)
-                else:
-                    self.params_captured_in_traffic[str(event["attributes"]["http_path"])][param] = [value]
-                try:
-                    self.params_captured_in_traffic[str(event["attributes"]["http_path"])][param] = \
-                        list(set(self.params_captured_in_traffic[str(event["attributes"]["http_path"])][param]))
-                except TypeError:
-                    pass
+                params_found[param] = value
+            if params_found:
+                if request["rsp_code"] not in [200, 201]:
+                    params_found["$RESPONSE_ERROR"] = request.get("rsp_error")
+                self.params_captured_in_traffic[request["rsp_code"]][str(event["attributes"]["http_path"])].append(params_found)
+                # if param in self.params_captured_in_traffic[request["rsp_code"]][str(event["attributes"]["http_path"])]:
+                #     # self.params_captured_in_traffic[str(event["attributes"]["http_path"]).lower()] = {}
+                #     #if value not in self.params_captured_in_traffic[request["rsp_code"]][str(event["attributes"]["http_path"])][
+                #     #    param]:
+                #     self.params_captured_in_traffic[request["rsp_code"]][str(event["attributes"]["http_path"])][param].append(
+                #             value)
+                # else:
+                #     self.params_captured_in_traffic[request["rsp_code"]][str(event["attributes"]["http_path"])][param] = [value]
+                # try:
+                #     self.params_captured_in_traffic[request["rsp_code"]][str(event["attributes"]["http_path"])][param] = \
+                #         list(set(self.params_captured_in_traffic[request["rsp_code"]][str(event["attributes"]["http_path"])][param]))
+                # except TypeError:
+                #     pass
+            #print(self.params_captured_in_traffic)
             cv_requests.append([request, event])
         print("\n\t\t......done collecting events data from APIShark")
         return cv_requests
@@ -751,7 +894,7 @@ def main():
     CloudvectorDAST(str(input_spec_one).strip(), str(input_spec_two).strip(), ce_host, ce_username, ce_password, config,
                     str(cover_only_diff).lower(), input_params_file, enable_fuzzing)
     print("\n Test generation complete! Making the pycode PEP complaint ( https://www.python.org/dev/peps/pep-0008/ ).....\n")
-    os.system("autopep8 --in-place --aggressive --aggressive tests/*")
+    os.system("autopep8 --in-place --aggressive --aggressive tests/*.py")
     print("\n\t\t..........done")
 
 
